@@ -27,6 +27,54 @@ from sphinx.util.nodes import make_refnode
 from sphinx.util.docfields import GroupedField, TypedField
 
 
+class DocRef(object):
+    """Represents a link to an RFC which defines an HTTP method."""
+
+    def __init__(self, base_url, anchor, section):
+        """Stores the specified attributes which represent a URL which links to
+        an RFC which defines an HTTP method.
+
+        """
+        self.base_url = base_url
+        self.anchor = anchor
+        self.section = section
+
+    def __repr__(self):
+        """Returns the URL which this object represents, which points to the
+        location of the RFC which defines some HTTP method.
+
+        """
+        return '{}#{}{}'.format(self.base_url, self.anchor, self.section)
+
+
+#: The URL of the HTTP/1.1 RFC which defines the HTTP methods OPTIONS, GET,
+#: HEAD, POST, PUT, DELETE, TRACE, and CONNECT.
+RFC2616 = 'http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html'
+
+#: The name to use for section anchors in RFC2616.
+RFC2616ANCHOR = 'sec'
+
+#: The URL of the RFC which defines the HTTP PATCH method.
+RFC5789 = 'http://tools.ietf.org/html/rfc5789'
+
+#: The name to use for section anchors in RFC5789.
+RFC5789ANCHOR = 'section-'
+
+#: Mapping from lowercase HTTP method name to :class:`DocRef` object which
+#: maintains the URL which points to the section of the RFC which defines that
+#: HTTP method.
+DOCREFS = {
+    'patch': DocRef(RFC5789, RFC5789ANCHOR, 2),
+    'options': DocRef(RFC2616, RFC2616ANCHOR, 9.2),
+    'get': DocRef(RFC2616, RFC2616ANCHOR, 9.3),
+    'head': DocRef(RFC2616, RFC2616ANCHOR, 9.4),
+    'post': DocRef(RFC2616, RFC2616ANCHOR, 9.5),
+    'put': DocRef(RFC2616, RFC2616ANCHOR, 9.6),
+    'delete': DocRef(RFC2616, RFC2616ANCHOR, 9.7),
+    'trace': DocRef(RFC2616, RFC2616ANCHOR, 9.8),
+    'connect': DocRef(RFC2616, RFC2616ANCHOR, 9.9)
+}
+
 HTTP_STATUS_CODES = {
     100: 'Continue',
     101: 'Switching Protocols',
@@ -153,6 +201,11 @@ class HTTPHead(HTTPResource):
     method = 'head'
 
 
+class HTTPPatch(HTTPResource):
+
+    method = 'patch'
+
+
 class HTTPPost(HTTPResource):
 
     method = 'post'
@@ -223,27 +276,14 @@ def http_statuscode_role(name, rawtext, text, lineno, inliner,
 
 def http_method_role(name, rawtext, text, lineno, inliner,
                      options={}, content=[]):
-    references = {
-        'options': '9.2',
-        'get': '9.3',
-        'head': '9.4',
-        'post': '9.5',
-        'put': '9.6',
-        'delete': '9.7',
-        'trace': '9.8',
-        'connect': '9.9'
-    }
     method = str(text).lower()
-    umethod = method.upper()
-    try:
-        sec = references[method]
-    except KeyError:
+    if method not in DOCREFS:
         msg = inliner.reporter.error('%s is not valid HTTP method' % umethod,
                                      lineno=lineno)
         prb = inliner.problematic(rawtext, rawtext, msg)
         return [prb], [msg]
-    url = 'http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html#sec' + sec
-    node = nodes.reference(rawtext, umethod, refuri=url, **options)
+    url = str(DOCREFS[method])
+    node = nodes.reference(rawtext, method.upper(), refuri=url, **options)
     return [node], []
 
 
@@ -267,20 +307,32 @@ class HTTPIndex(Index):
     localname = 'HTTP Routing Table'
     shortname = 'routing table'
 
+    def __init__(self, *args, **kwargs):
+        super(HTTPIndex, self).__init__(*args, **kwargs)
+
+        self.ignore = [[l for l in x.split('/') if l]
+            for x in self.domain.env.config['http_index_ignore_prefixes']]
+        self.ignore.sort(key=lambda x: -len(x))
+
+    def grouping_prefix(self, path):
+        letters = [x for x in path.split('/') if x]
+        for prefix in self.ignore:
+            if letters[:len(prefix)] == prefix:
+                return '/' + '/'.join(letters[:len(prefix) + 1])
+        return '/%s' % (letters[0] if letters else '',)
+
     def generate(self, docnames=None):
         content = {}
-        for method, routes in self.domain.routes.iteritems():
-            for path, info in routes.iteritems():
-                letter = path.split('/', 2)
-                try:
-                    first_letter = letter[1]
-                except IndexError:
-                    first_letter = letter[0]
-                entries = content.setdefault('/' + first_letter, [])
-                entries.append([
-                    method.upper() + ' ' + path, 0, info[0],
-                    http_resource_anchor(method, path), '', '', info[1]
-                ])
+        items = ((method, path, info)
+            for method, routes in self.domain.routes.iteritems()
+            for path, info in routes.iteritems())
+        items = sorted(items, key=lambda item: item[1])
+        for method, path, info in items:
+            entries = content.setdefault(self.grouping_prefix(path), [])
+            entries.append([
+                method.upper() + ' ' + path, 0, info[0],
+                http_resource_anchor(method, path), '', '', info[1]
+            ])
         content = content.items()
         content.sort(key=lambda (k, v): k)
         return (content, True)
@@ -298,6 +350,7 @@ class HTTPDomain(Domain):
         'post': ObjType('post', 'post', 'obj'),
         'get': ObjType('get', 'get', 'obj'),
         'put': ObjType('put', 'put', 'obj'),
+        'patch': ObjType('patch', 'patch', 'obj'),
         'delete': ObjType('delete', 'delete', 'obj'),
         'trace': ObjType('trace', 'trace', 'obj')
     }
@@ -308,6 +361,7 @@ class HTTPDomain(Domain):
         'post': HTTPPost,
         'get': HTTPGet,
         'put': HTTPPut,
+        'patch': HTTPPatch,
         'delete': HTTPDelete,
         'trace': HTTPTrace
     }
@@ -318,6 +372,7 @@ class HTTPDomain(Domain):
         'post': HTTPXRefRole('post'),
         'get': HTTPXRefRole('get'),
         'put': HTTPXRefRole('put'),
+        'patch': HTTPXRefRole('patch'),
         'delete': HTTPXRefRole('delete'),
         'trace': HTTPXRefRole('trace'),
         'statuscode': http_statuscode_role,
@@ -330,6 +385,7 @@ class HTTPDomain(Domain):
         'post': {},
         'get': {},
         'put': {},
+        'patch': {},
         'delete': {},
         'trace': {}
     }
@@ -404,7 +460,7 @@ class HTTPLexer(RegexLexer):
 
     tokens = {
         'root': [
-            (r'(GET|POST|PUT|DELETE|HEAD|OPTIONS|TRACE)( +)([^ ]+)( +)'
+            (r'(GET|POST|PUT|PATCH|DELETE|HEAD|OPTIONS|TRACE)( +)([^ ]+)( +)'
              r'(HTTPS?)(/)(1\.[01])(\r?\n|$)',
              bygroups(Name.Function, Text, Name.Namespace, Text,
                       Keyword.Reserved, Operator, Number, Text),
@@ -430,4 +486,5 @@ def setup(app):
         get_lexer_by_name('http')
     except ClassNotFound:
         app.add_lexer('http', HTTPLexer())
+    app.add_config_value('http_index_ignore_prefixes', [], None)
 
